   
# sim22.py
# Andrew Davison, ad@coe.psu.ac.th, July 2024
'''
  Simulate games with a 2x2 payoff matrix
  e.g. prisoners' dilemma

  Display points along a diagonal line.

  Example files:
    battle.txt, battle2.txt, battle27.txt, battle3.txt,
    battle4.txt, chicken.txt, freeMoney.txt, goalie.txt,
    pd.txt, pd2.txt, pd29.txt, stag.txt, stoplight.txt,
    takeShare.txt, wpennies.txt
'''

import random, math, sys
import matplotlib.pyplot as plt 
# import numpy as np
import gtUtils

NUM_SESSIONS = 10
NUM_ROWS = 2
NUM_TRIALS = 700
NUM_PLAYERS = 10



def plotPlayers(pts1, szs1, pts2, szs2):
  # scatter plots along a diagonal
  # P1
  xs1, ys1 = zip(*pts1)
  plt.scatter(xs1, ys1, s=szs1,  alpha=0.5, label="P1")  
  # P2
  xs2, ys2 = zip(*pts2)
  plt.scatter(xs2, ys2, s=szs2, alpha=0.5, label="P2") 

  plt.title(clabels[0])
  plt.xlabel(clabels[1])
  plt.ylabel(clabels[2])
  
  plt.axis('scaled')
  plt.grid()
  plt.xlim([-0.1, 1.1])
  plt.xticks([0, 0.2, 0.4, 0.6, 0.8, 1]) 
  # plt.xticks(np.arange(0, 1.1, 0.2)) 
  plt.ylim([-0.1, 1.1])
  plt.yticks([0, 0.2, 0.4, 0.6, 0.8, 1]) 
  # plt.yticks(np.arange(0, 1.1, 0.2)) 
  plt.legend()


# ------------

if len(sys.argv) != 2:
  print("Usage: python sim22.py <fnm>")
  sys.exit()

fnm = sys.argv[1]
clabels, payoffs = gtUtils.readPayoff(fnm)

if (len(payoffs) != 2) or not gtUtils.isSquare(payoffs):
  print("Payoff matrix must be 2x2")
  sys.exit()


plt.figure(figsize=(3.4*NUM_SESSIONS//NUM_ROWS, 4*NUM_ROWS))
plt.tight_layout()
plt.subplots_adjust(left=0.05)
gtUtils.setWindow(plt, fnm + ": " + clabels[0], 100,100)

for i in range(NUM_SESSIONS):
  plt.subplot(NUM_ROWS, NUM_SESSIONS//NUM_ROWS, i+1)
  pts1, szs1, pts2, szs2 = \
    gtUtils.playGames(payoffs, NUM_PLAYERS, NUM_TRIALS)
  # print("1--------------")
  # print(pts1)
  # print(szs1)
  plotPlayers(pts1, szs1, pts2, szs2)

plt.show()
