   
# sim33.py
# Andrew Davison, ad@coe.psu.ac.th, July 2024

'''
  Simulate games with a 2x2 payoff matrix
  e.g. Rock, paper, scissors

  Display points inside a simplex triangle using
  python-ternary and Barycentric coordinates.

  pip install python-ternary
  https://github.com/marcharper/python-ternary


  Example files:
    fictious.txt, rps3.txt, rps36.txt, rps37.txt
'''

import random, math, sys
import matplotlib.pyplot as plt 
import ternary, warnings
import gtUtils


NUM_SESSIONS = 10
NUM_ROWS = 2
NUM_TRIALS = 500
NUM_PLAYERS = 10



def plotPlayers(pts1, szs1, pts2, szs2):
  # ternary plot 
  fig, tax = ternary.figure(ax=plt.gca())
  # fig.set_size_inches(6, 5.5)
  
  # P1
  with warnings.catch_warnings():
    warnings.simplefilter("ignore", category=UserWarning)
    tax.scatter(pts1, s=szs1, alpha=0.5, label="P1")
  # P2
  with warnings.catch_warnings():
    warnings.simplefilter("ignore", category=UserWarning)
    tax.scatter(pts2, s=szs2, alpha=0.5, label="P2")
  
  tax.set_title(clabels[0])
  tax.bottom_axis_label(clabels[1])
  tax.right_axis_label(clabels[2])
  tax.left_axis_label(clabels[3])
  
  # Decoration
  tax.gridlines(multiple=0.1, color="gray")
  tax.ticks(axis='lbr', multiple=0.2, linewidth=1, 
                tick_formats="%.1f",  offset=0.025)
  tax.get_axes().axis('off')
  tax.legend()
 

# ----------------------

if len(sys.argv) != 2:
  print("Usage: python sim33.py <fnm>")
  sys.exit()

fnm = sys.argv[1]
clabels, payoffs = gtUtils.readPayoff(fnm)

if (len(payoffs) != 3) or not gtUtils.isSquare(payoffs):
  print("Payoff matrix must be 3x3")
  sys.exit()

plt.figure(figsize=(3.4*NUM_SESSIONS//NUM_ROWS, 4*NUM_ROWS))
plt.tight_layout()
plt.subplots_adjust(left=0.05)
gtUtils.setWindow(plt, fnm + ": " + clabels[0], 100,100)

for i in range(NUM_SESSIONS):
  plt.subplot(NUM_ROWS, NUM_SESSIONS//NUM_ROWS, i+1)
  pts1, szs1, pts2, szs2 = \
    gtUtils.playGames(payoffs, NUM_PLAYERS, NUM_TRIALS)
  plotPlayers(pts1, szs1, pts2, szs2)


plt.show()
