   
# sim44.py
# Andrew Davison, ad@coe.psu.ac.th, July 2024

'''
  Simulate games with a 4x4 payoff matrix

  Display points inside a tetrahedron using Barycentric coordinates.

  Example files:
    duoComp.txt, generals.txt, randomGame.txt, rowCol.txt

'''

import random, math, sys, itertools
import matplotlib.pyplot as plt 
import gtUtils


# NUM_SESSIONS = 3
# NUM_ROWS = 1
# NUM_TRIALS = 1000

NUM_SESSIONS = 10
NUM_ROWS = 2
NUM_TRIALS = 500

NUM_PLAYERS = 10

# Barycentric coords of vertices
A_BAR = [1, 0, 0, 0] 
B_BAR = [0, 1, 0, 0]
C_BAR = [0, 0, 1, 0]
D_BAR = [0, 0, 0, 1]
BARS = [A_BAR, B_BAR, C_BAR, D_BAR]

# Cartesian coords of vertices 
A_CART = [0, 0, 0]
B_CART = [1, 0, 0]
C_CART = [0.5, math.sqrt(3)/2, 0]
D_CART = [0.5, 0.28867513, 0.81649658]
CARTS = [A_CART, B_CART, C_CART, D_CART]


def tetraAxes(ax):         
  # plot tetrahedral axes and label verts
  axes = itertools.combinations(CARTS, 2)  # 6 axis lines
  for axis in axes:
    xyzs = list(zip(*axis))
    # print("xyzs", xyzs)
    ax.plot3D(xyzs[0], xyzs[1], xyzs[2], 'black')  
  labelVerts(ax) 


def labelVerts(ax):  
  # create labels of each vertices of the simplex
  pts = gtUtils.baryToCart(BARS, CARTS)
  # print("pts", pts)
  for i in range(len(pts)):
    pt = pts[i]
    ax.text(pt[0], pt[1], pt[2], str(BARS[i]), 
                       color="red", size=10)  # (x,y,z)



def plotPlayers(ax, pts1, szs1, pts2, szs2):
  # quadray plot 
  ax.set_title(clabels[0])
  ax.text2D(-0.1, -0.1, str(clabels[1:]))

  # scatter plots
  # P1
  cpts1 = gtUtils.baryToCart(pts1, CARTS)
  xyzs = list(zip(*cpts1))
  ax.scatter(xyzs[0], xyzs[1], xyzs[2], s=szs1, 
            alpha=0.5, label="P1")  
  # P2
  cpts2 = gtUtils.baryToCart(pts2, CARTS)
  xyzs = list(zip(*cpts2))
  ax.scatter(xyzs[0], xyzs[1], xyzs[2], s=szs2,
                alpha=0.5, label="P2") 
  ax.legend()


# ----------------------

if len(sys.argv) != 2:
  print("Usage: python sim44.py <fnm>")
  sys.exit()

fnm = sys.argv[1]
clabels, payoffs = gtUtils.readPayoff(fnm)

if (len(payoffs) != 4) or not gtUtils.isSquare(payoffs):
  print("Payoff matrix must be 4x4")
  sys.exit()

fig = plt.figure(figsize=(3.4*NUM_SESSIONS//NUM_ROWS, 4*NUM_ROWS))
plt.tight_layout()
plt.subplots_adjust(left=0.05)
gtUtils.setWindow(plt, fnm + ": " + clabels[0], 100,100)


for i in range(NUM_SESSIONS):
  ax = fig.add_subplot(NUM_ROWS, NUM_SESSIONS//NUM_ROWS, i+1, projection='3d')
  plt.axis('off')
  tetraAxes(ax)

  pts1, szs1, pts2, szs2 = \
    gtUtils.playGames(payoffs, NUM_PLAYERS, NUM_TRIALS)
  plotPlayers(ax, pts1, szs1, pts2, szs2)

plt.show()
